﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.Map;

namespace VisualizationLayerAnimatedRoute
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private BingRouteProvider routeProvider;
        private LocationCollection wayPoints;
        private ObservableCollection<AnimatedMapItem> animatedItems = new ObservableCollection<AnimatedMapItem>();
        private IEnumerator<Action<Action>> animationActions;
        private Storyboard storyboard = null;
        private bool animationStopped = false;
        private Color[] colors = new Color[]{Colors.Blue, Colors.Red, Colors.Green, Colors.Yellow};
        private int requestCounter = -1;
        private int portionCount = 10;

        public MainWindow()
        {
            InitializeComponent();
            this.wayPoints = this.GetSomePoints();

            this.routeProvider = new BingRouteProvider();
            this.routeProvider.ApplicationId = "AqaPuZWytKRUA8Nm5nqvXHWGL8BDCXvK8onCl2PkC581Zp3T_fYAQBiwIphJbRAK";
            this.routeProvider.MapControl = this.radMap;
            this.routeProvider.RoutingCompleted += this.RouteProvider_RoutingCompleted;

            this.wayPointsLayer.ItemsSource = this.wayPoints;

            this.animationLayer.ItemsSource = this.animatedItems;
        }

        private void MapMouseClick(object sender, MapMouseRoutedEventArgs eventArgs)
        {
            this.wayPoints.Add(eventArgs.Location);
            Debug.WriteLine(eventArgs.Location);
            this.pointsCount.Text = this.wayPoints.Count.ToString();
        }

        private void ClearRouteClicked(object sender, RoutedEventArgs e)
        {
            this.findRouteButton.IsEnabled = true;

            this.wayPoints.Clear();
            this.routeLayer.Items.Clear();
        }

        private void FindRouteClicked(object sender, RoutedEventArgs e)
        {
            this.routeLayer.Items.Clear();  
            if (this.wayPoints.Count <= 1)
                return;
          
            this.findRouteButton.IsEnabled = false;

            /////////////////// v.1.0
            //RouteRequest routeRequest = new RouteRequest();
            //routeRequest.Culture = new System.Globalization.CultureInfo("en-US");
            //routeRequest.Options.RoutePathType = RoutePathType.Points;

            //if (this.wayPoints.Count > 1)
            //{
            //    this.findRouteButton.IsEnabled = false;

            //    foreach (Location location in this.wayPoints)
            //    {
            //        routeRequest.Waypoints.Add(location);
            //    }
            //    this.routeProvider.CalculateRouteAsync(routeRequest);
            //}
            ///////////////////////////

            /// V 2.0

            List<LocationCollection> dividedList = this.DivideWayPoints();
            foreach (var list in dividedList)
            {
                RouteRequest routeRequest = new RouteRequest();
                routeRequest.Culture = new System.Globalization.CultureInfo("en-US");
                routeRequest.Options.RoutePathType = RoutePathType.Points;

                foreach (Location location in list)
                {
                    routeRequest.Waypoints.Add(location);
                }
                this.routeProvider.CalculateRouteAsync(routeRequest);
            }   
        }

        private List<LocationCollection> DivideWayPoints()
        {
            List<LocationCollection> list = new List<LocationCollection>();
            if (this.wayPoints.Count <= this.portionCount)
            {
                list.Add(this.wayPoints);
            }
            else
            {
                LocationCollection coll = new LocationCollection();
                foreach (var point in wayPoints)
                {
                    coll.Add(point);
                    if (coll.Count == this.portionCount)
                    {
                        Location lastLoc = coll[this.portionCount - 1];
                        list.Add(coll);
                        coll = new LocationCollection(){lastLoc};
                    }
                }
                list.Add(coll);
            }
            return list;
        }

        private void RouteProvider_RoutingCompleted(object sender, RoutingCompletedEventArgs e)
        {
            requestCounter++;
            this.findRouteButton.IsEnabled = true;

            RouteResponse routeResponse = e.Response as RouteResponse;
            if (routeResponse != null
                && routeResponse.Error == null)
            {
                if (routeResponse.Result != null
                    && routeResponse.Result.RoutePath != null)
                {
                    PolylineData routeLine = new PolylineData()
                    {
                        Points = routeResponse.Result.RoutePath.Points,
                        ShapeFill = new MapShapeFill()
                        {
                            Stroke = new SolidColorBrush(Colors.Red),
                            StrokeThickness = 5
                        }
                    };

                    this.routeLayer.Items.Add(routeLine);
                }
            }
            else 
            {
                MessageBox.Show(routeResponse.Error.ToString());
            }
        }

        private void AnimateRouteClicked(object sender, RoutedEventArgs e)
        {
            if (this.routeLayer.Items.Count > 0)
            {
                PolylineData routeLine = this.routeLayer.Items[0] as PolylineData;
                if (routeLine != null && routeLine.Points.Count > 1)
                {
                    this.SetIsEnabled(false);
                    this.animatedItems.RemoveAll();

                    AnimatedMapItem item = new AnimatedMapItem()
                    {
                        Caption = DateTime.Now.ToString(),
                        Location = routeLine.Points[0]
                    };
                    this.animatedItems.Add(item);

                    this.animationActions = this.AnimationSequence().GetEnumerator();
                    this.RunNextAction();
                }
            }

        }

        IEnumerable<Action<Action>> AnimationSequence()
        {
            if (this.routeLayer.Items.Count > 0)
            {
                PolylineData routeLine = this.routeLayer.Items[0] as PolylineData;
                if (routeLine != null && routeLine.Points.Count > 1)
                {
                    for (int i = 0; i < routeLine.Points.Count - 1; i++)
                    {
                        yield return this.AnimateItem(this.animatedItems[0], (Point)routeLine.Points[i], (Point)routeLine.Points[i + 1]);
                    }
                }
            }
        }

        private void RunNextAction()
        {
            if (animationActions.MoveNext())
            {
                animationActions.Current(RunNextAction);
            }
            else
            {
                this.SetIsEnabled(true);
            }
        }

        private Action<Action> AnimateItem(AnimatedMapItem item, Point from, Point to)
        {
            return completed =>
            {
                PointAnimation pointAnimation = new PointAnimation()
                {
                    Duration = TimeSpan.FromMilliseconds(300),
                    From = from,
                    To = to,
                };

                Storyboard.SetTarget(pointAnimation, item);
                Storyboard.SetTargetProperty(pointAnimation, new PropertyPath("Point"));
                
                this.storyboard = new Storyboard()
                {
                    AutoReverse = false,
                };

                storyboard.Completed += this.Storyboard_Completed;

                storyboard.Children.Add(pointAnimation);
                storyboard.Begin();
            };
        }

        private void Storyboard_Completed(object sender, EventArgs e)
        {
            if (!this.animationStopped)
            {
                this.RunNextAction();
            }
        }

        private void SetIsEnabled(bool enabled)
        {
            this.findRouteButton.IsEnabled = enabled;
            this.clearButton.IsEnabled = enabled;
            this.animateButton.IsEnabled = enabled;
            this.cancelAnimationButton.IsEnabled = !enabled;
        }

        private void CancelAnimationClicked(object sender, RoutedEventArgs e)
        {
            if (this.storyboard != null)
            {
                this.storyboard.Stop();
                this.SetIsEnabled(true);
            }
        }

        private LocationCollection GetSomePoints()
        {
            LocationCollection result = new LocationCollection();
            result.Add(new Location(42.7350352318869,23.2468128204346));
            result.Add( new Location(42.7328287008804,23.2509326934815));
            result.Add(new Location(42.7248214862528,23.265609741211));
            result.Add(new Location(42.7137862293283,23.2851791381836));
            result.Add(new Location(42.7057765567934,23.3090400695801));
            result.Add(new Location(42.7050827569902,23.3330726623536));
            result.Add(new Location(42.6939178212388,23.3342742919922));
            result.Add(new Location(42.6886815949656,23.3398532867432));
            result.Add(new Location(42.6805424875524,23.3513545989991));
            result.Add(new Location(42.6724329850468,23.3628950823954));
            result.Add(new Location(42.6644868089625,23.3741208466008));
            result.Add(new Location(42.65630813557,23.3857727050782));
            result.Add(new Location(42.651447505888,23.379077911377));
            result.Add(new Location(42.6442505514006,23.3739280700684));
            result.Add(new Location(42.6364844824272,23.3695507049561));
            result.Add(new Location(42.6241072801487,23.3652591705323));
            result.Add(new Location(42.6328220973397,23.3336734771729));
            result.Add(new Location(42.641409419098,23.3121299743653));
            result.Add(new Location(42.6535306794115,23.2831192016602));
            result.Add(new Location(42.6496799099986,23.2660388946534));
            result.Add(new Location(42.6650184434292,23.2596874237061));
            result.Add(new Location(42.677135102723,23.2496452331543));
            result.Add(new Location(42.6855901216197,23.2359981536866));
            result.Add(new Location(42.6985228120042,23.2300758361817));

            return result;
        }

        private void RadButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.radMap.Items.Count > 0)
            {
                this.radMap.Items.RemoveAt(0);
            }
        }
    }
}
